/**
 * Filename: disk_stats.h
 *
 * Description: Child class of the system_info class, this class
 * will gather information about the filesystem on the local 
 * system
 *
 * Original Author: Nathan Boeger
 *
 * $Id: disk_stats.cpp,v 1.5 2002/05/08 06:11:06 nathan Exp $
 *
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include <syslog.h>
#include <sys/uio.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/param.h>
#include <sys/ucred.h>

/* my includes */
#include "flog.h"
#include "khneia.h"
#include "khneia_protocol.h"
#include "khneia_sender.h"
#include "client_sys_conf.h"
#include "system_info.h"
#include "disk_stats.h"

/*
 * Convert statfs returned filesystem size into BLOCKSIZE units.
 * Attempts to avoid overflow for large filesystems. This is taken
 * from df !! aah.. open source is great :-)
 */
#define fsbtoblk(num, fsbs, bs) \
        (((fsbs) != 0 && (fsbs) < (bs)) ? \
        (num) / ((bs) / (fsbs)) : (num) * ((fsbs) / (bs)))


disk_stats::disk_stats()
{
	error = K_BAD;
	sys = NULL;
}

disk_stats::disk_stats(khneia_header_pkt *h, client_sys_conf *s)
{
	if( (h) && (s) )
	{
		cnt = 0;
		sys = s;
		info = new disk_info_pkt;

		if( info )
		{
			info->header.length = sizeof(struct disk_info_pkt);
			info->header.sequence = h->sequence;
			info->header.type = h->type;
			error = K_GOOD;
			// set the header
			header = (struct khneia_header_pkt*)info;
		}

	}
	else
	{
		K_LOG("Constructor error, invalid data !");
	}

}

/*
 * run
 *
 * not much to do but call the proper methods to gather the needed info
 */

int disk_stats::run()
{
	getDiskInfo();

	return(error);
}

/*
 * getDiskInfo
 *
 * For this we will be using the getfsstat BSD system call. Once it
 * returns it will tell us how many statfs structs we have. ALl we
 * need to do is just loop through them and extract our needed info.
 */
int disk_stats::getDiskInfo()
{
	size_t len, cnt, plen;
	char *buf, *next, *packet, *nextpkt;
	struct statfs *fs;
	disk_info *dinfo;

	/* call it once to get the number of structs needed */
	cnt = getfsstat(NULL, (sizeof(struct statfs)), 0);
	if( (info) && (error != K_BAD) && (cnt > 0) )
	{
		/* now calculate how large the packt will have to be*/
		plen = (sizeof(struct disk_info) * cnt) + sizeof(struct khneia_header_pkt);
		packet = (char *)malloc(plen);

		/* now how much space we need for the buf */
		len = sizeof(struct statfs) * cnt ;
		buf = (char *)alloca(len);

		if( (buf) && (packet) )
		{
			/* copy over the old values */
			memcpy(packet, info, sizeof(struct disk_info_pkt) );
			delete info;
			/* now rest our global struct addresses !! */
			info = (struct disk_info_pkt *)packet;
			info->header.length = plen;
			header = (struct khneia_header_pkt *)packet;

			/* get the values and then add them to the packet */
			if( (getfsstat((struct statfs *)buf, len, 0)) > 0 )
			{
				next = buf;
				/* advance the buffer past the header info */
				nextpkt = packet + sizeof(struct disk_info_pkt);
				while( next < (buf + len) )
				{
					fs = (struct statfs *)next;
					dinfo = (struct disk_info *)nextpkt;

					/* zero out our buffers first */
					memset(dinfo, 0, sizeof(struct disk_info));   
					strncpy(dinfo->fstype, fs->f_fstypename, MAX_FS_LEN);
					strncpy(dinfo->mount, fs->f_mntonname, MAX_MOUNT_LEN);

					/* used f_bavial since its more accurate than f_bfree */
					dinfo->size = fsbtoblk(fs->f_blocks, fs->f_bsize, REPORT_UNIT);
					dinfo->free = fsbtoblk(fs->f_bavail, fs->f_bsize, REPORT_UNIT);
					dinfo->used = fsbtoblk( (fs->f_blocks - fs->f_bfree), fs->f_bsize, REPORT_UNIT);

					/* now set network byte order */
					H_TO_NWL(dinfo->size);
					H_TO_NWL(dinfo->free);
					H_TO_NWL(dinfo->used);

					/* advance our pointers */
					next += (sizeof(struct statfs) );
					nextpkt += (sizeof(struct disk_info) );
				}
			}
			else
			{
				error = K_BAD;
				K_ERR_LOG("failed call to getfsstat: ");
			}
		}
		else
		{
			error = K_BAD;
			K_ERR_LOG("failed call to getfsstat: ");
		}

	}
	else
	{
		error = K_BAD;
		K_LOG("invalid data ");
	}
	return(error);
}

/*
 * Just remove our info struct
 */
disk_stats::~disk_stats()
{
	free(info);
}


