/**
 * Filename: system_stats.cpp
 *
 * Description: Base class for all the system stats, this info will
 * compose the heart beat packets. This is derived from the system_info
 * class and will pride:
 *    1) CPU load information
 *    2) Memory usage information
 *    3) Swap usage information
 *
 * Original Author: Nathan Boeger
 *
 * $Id: system_stats.cpp,v 1.5 2002/04/17 05:35:48 nathan Exp $
 *
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/sysctl.h>
#include <sys/param.h>
#include <sys/vmmeter.h>
#include <sys/conf.h>
#include <fcntl.h>
#include <vm/vm_param.h>
#include <machine/asnames.h>
#include <kvm.h>
#include <nlist.h>

/* my includes */
#include "flog.h"
#include "khneia.h"
#include "khneia_protocol.h"
#include "khneia_sender.h"
#include "client_sys_conf.h"
#include "system_info.h"
#include "system_stats.h"

// needed macros
#define CNT            (3)
#define pagetok(size)  ((size) << pageshift)
#define X_CNT          (0)    

/*
 * default constructor
 *
 * just flag the erorr
 */
system_stats::system_stats()
{
	error = K_BAD;
	info = NULL;
}

system_stats::system_stats(khneia_header_pkt *h, client_sys_conf *s)
{
	if( (h) && (s) )
	{
		sys = s;
		info = new system_stats_pkt;

		if( info )
		{
			info->header.length = sizeof(struct system_stats_pkt);
			info->header.sequence = h->sequence;
			info->header.type = h->type;
			error = K_GOOD;
			// set the header !
			header = (struct khneia_header_pkt *)info;
		}

	}
	else
	{
		K_LOG("Constructor error, invalid data !");
	}
}



/*
 * run
 *
 * All we need to do is call the methods that will assemble the packet
 */

int system_stats::run()
{
	setLoadInfo();
	setMemoryInfo();

	/* now conver to network byte ordering */
	H_TO_NWL(info->mem_free);
	H_TO_NWL(info->mem_total);
	H_TO_NWL(info->mem_used);
	H_TO_NWL(info->swap_free);
	H_TO_NWL(info->swap_total);
	H_TO_NWL(info->swap_used);

	return(error);
}


/*
 * setLoadInfo
 *
 */
int system_stats::setLoadInfo()
{
	double d[CNT];

	/* go and get the types */
	if( getloadavg(d, CNT) > 0 )
	{
		snprintf(info->load_0, MAX_LOAD_LEN, LOAD_FORMAT, d[0] );
		snprintf(info->load_1, MAX_LOAD_LEN, LOAD_FORMAT, d[1] );
		snprintf(info->load_2, MAX_LOAD_LEN, LOAD_FORMAT, d[2] );
	}
	else
	{
		K_LOG("falied to getload()");
	}

	return(error);
}

/*
 * setMemoryInfo
 *
 * This is a bit ugly, cause we  use the KVM lib. Since SUN now
 * does not have KVM anymore.. not sure what to do about solaris 
 */
int system_stats::setMemoryInfo()
{
	int mib[2], err = K_GOOD, pagesize, pageshift, i;
	kvm_t *kd;
	size_t len;
	struct kvm_swap kswap;
	struct vmmeter vm;
	static struct nlist nlst[] = { { "_cnt"}, { 0}};

	/* quickly calculate our pageshift */
	pagesize = getpagesize(); 
	pageshift = 0;
	i = pagesize;
	while( i > 1 )
	{
		i >>= 1;
		pageshift++;
	}
	pageshift -= LG1024;

	/*
	 * First get the total memory on the system
	 */
	mib[0] = CTL_HW; 
	mib[1] = HW_PHYSMEM;
	len = sizeof(int);
	if( sysctl(mib, 2, &info->mem_total, &len, NULL, 0) >= 0 )
	{
		info->mem_total /= KILOBYTE;
		/*
		 * Now we get the swap information, we need to divide by kilobyte
		 * so we have nice megs for our useage.  
		 */
		kd = kvm_open(NULL, NULL, NULL, O_RDONLY, "kvm_open");  
		if( kd != NULL )
		{
			if( kvm_getswapinfo(kd, &kswap, 1, NULL) >= 0 )
			{
				info->swap_total = kswap.ksw_total * pagesize / KILOBYTE;
				info->swap_used = kswap.ksw_used * pagesize / KILOBYTE;
				info->swap_free = (info->swap_total - info->swap_used );
				/*
				 * now we have to get the memory usage, this will have to read out from the
				 * kernel via kvm and get the vmmeter struct, that has all we need. note that
				 * agian we need to make our answer in terms of megs  
				 */
				if( kvm_nlist(kd, nlst) >= 0 )
				{
					if( kvm_read(kd, nlst[X_CNT].n_value, (char *)&vm, sizeof(struct vmmeter)) >= 0 )
					{
						info->mem_used = pagetok(vm.v_active_count);
						info->mem_free = (info->mem_total - info->mem_used );
					}
					else
					{
						err = K_BAD;
						K_ERR_LOG("KVM read Error: ");
					}

				} // if(kvm_nlist)
				else
				{
					err = K_BAD;
					K_ERR_LOG("KVM nlist Error: ");
				}

			} // if(kvm_getswap)
			else
			{
				err = K_BAD;
				K_ERR_LOG("KVM getswap Error: ");
			}

		} // if sysctl
		else
		{
			err = K_BAD;
			K_ERR_LOG("sysctl() Error: ");
		}

		/* close the kvm */
		kvm_close(kd);    
	}
	else
	{
		err = K_BAD;
		K_ERR_LOG("KVM cannot open Error: ");
	}

	return(err);
}


/*
 * just delete our info packet
 */
system_stats::~system_stats()
{
	delete info;
}
