/**
 * Filename: master_listener.cpp
 *
 * Description: Class implementation file for the master_listener module
 *
 * Original Author: Nathan Boeger
 *
 * $Id: master_listener.cpp,v 1.17 2002/05/07 06:50:52 nathan Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/resource.h>

// my includes 
#include "flog.h"
#include "khneia.h"
#include "master_sys_conf.h"
#include "khneia_protocol.h"
#include "master_listener.h"
#include "db_agent.h"
#include "db_hinfo.h"
#include "db_system_stats.h"
#include "db_disk_info.h"


int master_listener::forkproc = 0;

master_listener::master_listener()
{
	sys_conf = NULL;
	error = K_BAD;
	maxfork = 0;
	sfd = 0;
}

/*
 * Alternate constuctor
 *
 * Set our conf file parser, if not good then flag the error
 */
master_listener::master_listener(master_sys_conf *s)
{
	sfd = 0;

	if( s )
	{
		sys_conf = s;
		maxfork = sys_conf->getMAXFork();
		error = K_GOOD;
	}
	else
	{
		// would be bad anyway, but for clarity set it
		error = K_BAD;
	}
}

/*
 * run
 *
 * Run the listener, all we have to do is bind to the socket and
 * once we recieve a connection just create the proper module to
 * service the request
 */
int master_listener::run()
{
	int nsfd = 0, n = 1;
	size_t len;
	char *buff, *base;
	struct sockaddr_in sock, client;
	struct khneia_header_pkt header;


	if( (sfd = socket(AF_INET, SOCK_STREAM, 0)) > 0 )
	{
		/* 
		 * check the sys_conf module to make sure we can get the
		 * valid opts
		 */
		if( sys_conf->getError() != K_BAD )
		{
			memset(&sock, 0, sizeof(struct sockaddr_in) );
			sock.sin_family = AF_INET;
			sock.sin_port = htons(sys_conf->getListenPort());

			/*
			 * Set our opts 
			 */
			if( setsockopt(sfd, SOL_SOCKET, SO_REUSEADDR, &n, sizeof(int)) >= 0 )
			{
				/*
				 * Bind to the socket, but do not be nice.. just loop untill we get it !
				 */
				while( bind(sfd, (struct sockaddr *) &sock, sizeof(struct sockaddr_in)) < 0 )
				{
					K_ERR_LOG("Cannot bind !");
					sleep(3);
				}

				// set signal handler
				signal(SIGCHLD, child_cleanup);

				while( 1 )
				{
					if( listen(sfd, MAX_CONNECTIONS) >= 0 )
					{

						len = sizeof(struct sockaddr_in);
						// now accept the socket
						nsfd = accept(sfd, (struct sockaddr *)&client, &len);

						// read from it, check the size read
						memset(&header, 0, sizeof(struct khneia_header_pkt));
						len = recv(nsfd, &header, sizeof(struct khneia_header_pkt), 0);

						NW_TO_HL(header.length);
						NW_TO_HL(header.sequence);

						// create our buff to handle the data 
						buff = (char *)malloc(header.length);
						if( buff != NULL )
						{
							base = buff;
							// copy over the header
							memcpy(buff, &header, sizeof(struct khneia_header_pkt) );
							// a dvance our pointer
							buff += sizeof(struct khneia_header_pkt);
							// get the rest
							len = recv(nsfd, buff, header.length, 0);
							/*
							 * Check to see if we can fork ! if not then we
							 * just call sleep untill one of the children exits
							 */
							if( forkproc >= maxfork )
							{
								/*
								 * Just call the clean  up and then when it
								 * returns.. we can fork, since we do not
								 * set the WNOHANG
								 */
								child_cleanup(0);
							}

							if( fork() == 0 )
							{
								handle_pkt((struct khneia_header_pkt *)base);
							}

							forkproc++;
							shutdown(nsfd, SHUT_RDWR);
							free(base);
						} // if (buff )

					} // if( listen)
					else
					{
						K_LOG("MASTER RUN ERROR listen failed");
					}
				} // while (1) 

				shutdown(sfd, SHUT_RDWR);
			} // if(setsockopts)
			else
			{
				K_LOG("MASTER RUN ERROR setsockopts failed");
			}
		} // sys_conf
		else
		{
			K_LOG("The sysconf module has problems :-( \n");
		}
	} // if (socket)
	else
	{
		K_LOG("Socket returned error : ");
	}

	return(error);
}

int master_listener::handle_pkt(struct khneia_header_pkt *head)
{
	db_agent *db = NULL;

	if( head )
	{
		// deal with each packet
		switch( head->type )
		{
			// now deal with the packet
			case HOST_INFO_PKT:
				db = new db_hinfo((struct khneia_header_pkt *)head, sys_conf);
				break;

			case SYSTEM_STATS_PKT:
				db = new db_system_stats((struct khneia_header_pkt *)head, sys_conf);
				break;

			case DISK_INFO_PKT:
				db = new db_disk_info((struct khneia_header_pkt *)head, sys_conf);
				break;

			default:
				db = NULL;
				break;
		}

		if( db )
		{
			db->run();
			delete db;
		}

		db = NULL;

	}

	exit(0);

	// never reached....
	return(error);
}

/*
 * close 
 *
 * This will close the socket and clean up before
 * returing
 */
int master_listener::close()
{
	shutdown(sfd, SHUT_RDWR);
	return(error);
}

/*
 * child_cleanup will gather the info and then
 * decrament the forkproc
 */
void master_listener::child_cleanup(int sig)
{
	/* just discard for now.. add more code later if needed */
	waitpid(-1, (int *)NULL, 0);
	forkproc--;
}

/*
 * No dynamic memory yet.. 
 */
master_listener::~master_listener()
{

}


