/*
 * Filename: khneia.sql
 * Description: This file holds all the needed SQL statements used in the
 * khneia system
 *
 * $Id: khneia.sql,v 1.3 2002/05/08 06:11:07 nathan Exp $
 */

/* 
 * clients table
 * this will store all the information for each client. Note that the clients
 * are unique by thier hostname. This can be a FQDN so be carefull !
 */

DROP SEQUENCE new_client_id;

CREATE SEQUENCE new_client_id START 100;

DROP TABLE clients;

CREATE TABLE clients (
   client_id	  int4 DEFAULT NEXTVAL('new_client_id'), -- This is the unique node ID
   hostname	     CHAR(32),  -- Same as MAX_HOSTNAME_LEN   
   primary key(hostname)
);

/*
 * host_info
 * this will store all the host related information
 */
DROP TABLE host_info;

CREATE TABLE host_info(
   client_id      int4 NOT NULL,
   osinfo         CHAR(128), -- Same as MAX_OSINFO_LEN
   khneia_ver     CHAR(32),  -- Same as MAX_VERSION_LEN
   def_route      CHAR(16),  -- Since an IP can have at most 16 chars
   add_time       timestamp DEFAULT  CURRENT_TIMESTAMP -- Get the time this reccord was added
);

/*
 * disk_info table
 * This will store all the information about the systems disk. All the information
 * will be indexed by client_id
 */
DROP TABLE disk_info;

CREATE TABLE disk_info (
   client_id      int4 NOT NULL, -- client for which we are enterning
   mount          CHAR(128),     -- mount point for this file system
   fstype         CHAR(32),      -- fs type (ufs, proc, ext etc.. )
   total_size     int4,
   total_used     int4,
   total_free     int4,
   add_time       timestamp DEFAULT  CURRENT_TIMESTAMP -- Get the time this reccord was added
);


/*
 * network_info table
 * This table will store all the information. The inet address is used as a 
 */
DROP TABLE network_info;

CREATE TABLE network_info (
   client_id     int4 NOT NULL, -- client for which we are enetering
   dev           CHAR(16),      -- ethernet deivce 
   mac           CHAR(32),      -- ethernet mac address
   ipaddr        CHAR(16),      -- IP address as a string
   netmask       CHAR(16),      -- netmask 
   add_time       timestamp DEFAULT  CURRENT_TIMESTAMP -- Get the time this reccord was added
);

/*
 * system_stats table
 * This table will keep all system stats info. Such as mem, cpu and swap usage.
 *
 */ 
DROP TABLE system_stats;

CREATE TABLE system_stats (
   client_id   int4 NOT NULL, -- client for which we are entering
   load_0      float4,         -- load 0 value
   load_1      float4,
   load_2      float4,
   mem_total   int4,
   mem_used    int4,
   mem_free    int4,
   swap_total  int4,
   swap_used   int4,
   swap_free   int4,
   add_time    timestamp DEFAULT  CURRENT_TIMESTAMP -- Get the time this reccord was added
);



/*
 * poll table
 * This will hold entries for which systems we need to poll and which packets we 
 * need to request. Note that we will keep a status level, this level will be an
 * int starting at 0 -> N where N is the max level and N is > 1. This level 
 * is set to 0 initally and set to 1 when the client is polled. Once the client
 * responds then it will be set back to 0 (for packets with the sequence set) 
 * However this number will increase under these cercumstances:
 *
 *  I) when the client failes to respond prior to the next polling
 *
 * II) the client was unreachable or a network error occured
 *
 * This status value increases until  the max N value is reached (listed in the
 * threshold talble as err_level). At that point then the client will no longer 
 * be polled, untill its value is below its max N, or the client reponds to a request. 
 */

DROP TABLE poll;

CREATE TABLE poll (
   client_id   int4 NOT NULL, -- client id to poll
   status      int4 NOT NULL, -- what is the current status    
   pkt_type    int4 NOT NULL,  -- what type of packet to request
   PRIMARY KEY (client_id, pkt_type) -- only 1 packet type per host !
);


/*
 * threshold
 * This table will hold the clients thresholds. These are set as limits for the
 * particular max value.
 *
 */

DROP TABLE threshold;

CREATE TABLE threshold (
   client_id   int4 NOT NULL,   -- limits for this client
   err_level   int4 NOT NULL,   -- what is the max timeout limit (N)
   mem_usage   int4 NOT NULL,   -- what is the max mem_usage (percent)
   cpu_usage   float4 NOT NULL, -- what is the max load before an error ?
   swap_usage  int4 NOT NULL,   -- what is the max swap usage (percent)
   PRIMARY KEY (client_id)      -- each client can only be listed once
);

/*
 * poll_log table
 * This table will store the replies from the clients. Basicly so that
 * a systems history can be traced.
 */
 
DROP TABLE poll_log;

CREATE TABLE poll_log (
    client_id  int4 NOT NULL, -- store this clients info
    pkt_type   int4 NOT NULL, -- what type of packet was sent ?
    time_sent  timestamp,     -- what time was this packet sent ?
    time_reply timestamp,     -- when was the reply ?
    sequence   int4,          -- what was the sequence ?
    err_level  int4,          -- what was the error level ?
    notes      CHAR(64)       -- simple breif notes if any
);

/*
 * pkt_sequence 
 * This will keep track of the packets, note that this sequence will start
 * at 1, and then when the MAX sequence is reached it will cycle back to 0.
 * Since we need to support 4-byte ints we set MAX to 999999999
 *
 */

DROP SEQUENCE pkt_sequence;

CREATE SEQUENCE pkt_sequence INCREMENT 1  MINVALUE 0 MAXVALUE 999999999 START 1 CYCLE;


/*
 * packets table
 * This table will have the names / int values for our packets that khneia will
 * use.
 */
DROP TABLE packet;

CREATE TABLE packets (
   pkt_id      int4 NOT NULL,   -- INT value for this packet
   pkt_name    CHAR(16),        -- Short name for this packet
   pkt_notes   CHAR(128),       -- Any notes with this packet
   PRIMARY KEY (pkt_id)         -- only 1 type of packet id is allwed
);
